#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

// how many rows there are in the picture
#define ROWS 100

// width in pixels of a narrow bar
#define MULT 4

// size of border to either side of the barcode
#define BORDER 10

// the follow functions provide some infrastructure for building the codes
int size = 0, used = 0;
char *buffer = NULL;

void add_char (char c)		// c should be 0 or 1
{
	if (used == size)
	{
		size += 128;
		buffer = realloc(buffer,size);
	}
	buffer[used++] = c;
}

void wide_bar ()     { add_char(1); add_char(1); }
void narrow_bar ()   { add_char(1); }
void wide_space ()   { add_char(0); add_char(0); }
void narrow_space () { add_char(0); }

void start_code ()
{
	narrow_bar(); narrow_space();
	narrow_bar(); narrow_space();
}

void finish_code ()
{
	wide_bar(); narrow_space(); narrow_bar();
}

// output as PBM file
void gen_pbm ()
{
	int i,j,k;
	printf("P1\n");
	printf("%d %d\n",used*MULT + 2*BORDER,ROWS);
	for (i = 0; i < ROWS; i++)
	{
		for (k = 0; k < BORDER; k++) printf("0 ");
		for (j = 0; j < used; j++)
			for (k =0; k < MULT; k++) printf("%d ",buffer[j]);
		for (k = 0; k < BORDER; k++) printf("0 ");
		printf("\n");
	}
}

// pipe output into "display" command
void display ()
{
	int p[2];
	pipe(p);
	if (fork())
	{
		if (fork())
		{
			close(p[0]); close(p[1]);
			wait(NULL); wait(NULL);
		}
		else
		{
			close(p[0]); dup2(p[1],1); gen_pbm(); exit(0);
		}
	}
	else
	{
		close(p[1]);
		dup2(p[0],0);
		execlp("display","display",NULL);
	}
}

// Here is the code your should be modifying. The function encode() takes
// two arguments, which are values from 0..9 (not '0'..'9'), and which should
// be encoded by interleaved bars and spaces using the functions narrow_bar(),
// narrow_space() etc.

void encode (char c, char d)
{
	// to be completed
}

// The main program, which you will not need to modify.
int main (int argc, char **argv)
{
	// check whether the arguments make sense
	char *c;
	if (argc != 2)
	{
		fprintf(stderr,"usage: barcode <digits>\n");
		return 1;
	}
	for (c = argv[1]; *c; c++)
		if (*c < '0' || *c > '9')
		{
			fprintf(stderr,"non-digit in input; exiting\n");
			return 2;
		}
	if (strlen(argv[1]) % 2) 
	{
		fprintf(stderr,"odd number of digits; exiting\n");
		return 3;
	}
	
	// generate the barcode
	start_code();
	for (c = argv[1]; *c; c += 2) encode(c[0]-'0',c[1]-'0');
	finish_code();

	// output the barcode graphically
	display();
	return 0;
}
